//       1         2         3         4         5         6         7
// 4567890123456789012345678901234567890123456789012345678901234567890123456789

package la.la;
// ----------------------------------------------------------------------------

/* Copyright (c) 2018 Lloyd Allison.

   Reference: 
     Lloyd Allison, "Coding Ockham's Razor", Springer, 2018.

   This file is free software: you can redistribute it and/or modify it
   under the terms of the GNU General Public License (GPL) as published
   by the Free Software Foundation, either version 3 of the License, or
   (at your option) any later version.

   This file is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this file. If not, see http://www.gnu.org/licenses/
 */
// ----------------------------------------------------------------------------

import java.applet.*;       // contains class Applet
import java.net.*;          // contains URL
import java.awt.*;          // contains TextArea, Button, FlowLayout
import java.awt.event.*;    // contains WindowListener, ActionEvent
import java.io.*;           // contains InputStream, OutputStream

public class FPapplet extends Applet
 { private int initialisations = 0;
   int printLimit = 2000; // default value (in chars);

   public void init()                                   // Applet requires init
    { final int height = getHeight(), width = getWidth();
      setBackground( new Color(0xee, 0xee, 0xee) );
      FlowLayout mngr = new FlowLayout(FlowLayout.CENTER);
      mngr.setHgap(2); mngr.setVgap(2);
      setLayout( mngr );

      final Button b1   = new Button("run");                             // run
      add(b1);    // NB has size 0x0 until added!
      final int buttonHt = b1.getHeight();
      remove(b1); // but I want to add it elsewhere
      final TextArea tIp = textAreaSize(width-4, (height-buttonHt-8)/2);
      final TextArea tOp = textAreaSize(width-4, (height-buttonHt-8)/2);
      final PrintStream opSt = new PrintStream(textArea2OutputStream(tOp));

      b1.addActionListener(                                              // run
         new ActionListener()
          { public void actionPerformed(ActionEvent a)
             { int ch;
               InputStream ipSt = textArea2InputStream( tIp );
               try{ tOp.setText("");
                  opSt.println( "-- Lambda, parsing --" );
                  Syntax syn = new Syntax( new Lexical(ipSt) );
                  Expression e = syn.exp();  // parse a program
               // opSt.println("-- Expression --");
               // opSt.println( e.toString() );
                  opSt.println("-- Running --");
                  Value ans = e.defer(Environment.initial);
                  ans.print(opSt); opSt.println();
               }catch(Exception e) { opSt.println(e.getMessage()); };
               opSt.println("-- Done. Lambda, Sept. 2007, 2012 --");
          }  } );

      Button home = null;                                               // home
      try{ final URL homeURL = new URL("http://www.allisons.org/ll/");
           home = new Button("home");
           home.addActionListener(
              new ActionListener()
               { public void actionPerformed(ActionEvent a)
                  { opSt.println("-- http://www.allisons.org/ll/ --");
                    getAppletContext().showDocument( homeURL, "_top" );
               }  } );
      } catch(MalformedURLException u) { }

      this.add(tIp);  this.add(b1);
      if( home != null ) this.add(home);
      this.add(tOp);
      // this.add("ip", tIp);  this.add("b1", b1);
      // if( home != null ) this.add("home", home);
      // this.add("op", tOp);

      String printLimitStr = getParameter("PRINTLIMIT");
      if(printLimitStr != null) printLimit = Integer.parseInt(printLimitStr);

      final String inFileName = getParameter("INPUT");   //maybe a file to load
      if( inFileName != null )                        //into the input TextArea
      try{ tIp.setText(urlContents(relativeURL(getDocumentBase(), inFileName)));
         }
      catch(Exception e)
        { opSt.print( "Error in loading [" + inFileName + "]\n"
                    + e.toString() ); }

    }//init


   private TextArea textAreaSize(int width, int height)
   // Return a TextArea that will fit within a (width x height) rectangle.
    { TextArea ta = new TextArea(1,1);
      this.add(ta); // else size is 0x0
      final Dimension d10 = ta.getPreferredSize(10, 10),
                      d20 = ta.getPreferredSize(20, 20);
      final int ta10w = d10.width,  ta20w = d20.width,
                ta10h = d10.height, ta20h = d20.height;
      final int t0w = 2*ta10w - ta20w,
                t0h = 2*ta10h - ta20h,  //scrollbars etc
                colW = (ta20w-ta10w+9) / 10,   //one col
                rowH = (ta20h-ta10h+9) / 10;   //one row
      this.remove(ta);
      return new TextArea( (height-t0h-rowH+1) / rowH,   //rows
                           (width -t0w-colW+1) / colW ); //cols
    }//textAreaSize


   private InputStream textArea2InputStream(final TextArea ta)
    { return new InputStream()
       { String s = ta.getText();  int inPtr=0;
         public int read()  //minimum implementation of an InputStream
          { if( inPtr >= s.length() ) return -1;
            else { inPtr++; return s.charAt(inPtr-1); }
          }//read
         public int available() { return s.length()-inPtr; }
       };//InputStream
    }//textArea2InputStream

   private OutputStream textArea2OutputStream(final TextArea t)
    { return new OutputStream()
       { TextArea ta = t;
         int nChars = 0;
         public void write(int b) //minimum implementation of an OutputStream
          { byte[] bs = new byte[1]; bs[0] = (byte) b;
            ta.append(new String(bs));
            nChars++;
            if( nChars > printLimit )
             { nChars = 0;
               throw new RuntimeException("\nprint limit reached, ?looping?");
             }
          }//write
       };//OutputStream
    }//textArea2OutputStream


   private URL relativeURL(URL base, String relPath)
   throws MalformedURLException
   // Basically base/filename, but may chop a last filename off base first,
   // e.g., see class Applet, getDocumentBase() etc.
    {  String db = base.toString();
       int pathEnd = db.lastIndexOf(File.separatorChar);  // i.e. \ or /
       return new URL( db.substring(0, 1+pathEnd) + relPath );
    }//relativeURL(,)

   private String urlContents(URL url) throws IOException
   // Read a file at a URL and return its contents as a String
    { InputStream ins = url.openStream();
      StringBuffer sb = new StringBuffer();  int ch;
      while( (ch = ins.read()) >= 0 ) sb.append( (char)ch );
      return sb.toString();
    }//urlContents

 }//FPapplet class   http://www.allisons.org/ll/

// Lambda, (c) Lloyd Allison, Sept. 2007, 2012.
