//       1         2         3         4         5         6         7
// 4567890123456789012345678901234567890123456789012345678901234567890123456789

package mml;

import la.la.Value;
import la.maths.Vector;
import la.maths.Vector.Slice;
// ----------------------------------------------------------------------------

/* Copyright (c) 2018 Lloyd Allison.

   Reference: 
     Lloyd Allison, "Coding Ockham's Razor", Springer, 2018.

   This file is free software: you can redistribute it and/or modify it
   under the terms of the GNU General Public License (GPL) as published
   by the Free Software Foundation, either version 3 of the License, or
   (at your option) any later version.

   This file is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this file. If not, see http://www.gnu.org/licenses/
 */
// ----------------------------------------------------------------------------

/** A simple model of positive integers,&nbsp;&gt;0,
    where pr(n)&nbsp;=&nbsp;1/(n(n+1)).
    Note that &sum;<SUB>n&gt;0</SUB>&nbsp;1/(n(n+1))
    = &sum;<SUB>n&gt;0</SUB>{1/n&nbsp;-&nbsp;1/(n+1)}
    =&nbsp;1&nbsp;- 1/2 + 1/2 - 1/3 + 1/3 ... =&nbsp;1.
    The (trivially) fully parameterised model is {@link Int1.M}.<BR>
    Better models of integers, for most purposes, are
    {@link MML#WallaceInt0}, {@link MML#logStar0},
    {@link MML#Poisson0} and {@link MML#Geometric0}, but <CODE>Int1</CODE>
    might be useful and does make a simple case study.  */
public class Int1 extends Discretes                                     // Int1
 { /** dp must be triv.  */
   public Int1(Value dp)
     { super(dp); if( dp != triv ) error( ", dp=" + dp); }//constructor

   /** Sufficient statistics 'ss' are the data-set itself. */
   public Value stats(Vector ds, int lo, int hi)
    { return ds.slice(lo, hi); }

   public Value stats(boolean add, Value ss0, Value ss1)
    { return ((Vector) ss0).combine(add, (Slice) ss1); }

   public M sp2Model(double msg1, double msg2, Value sp)
    { return new M(msg1, msg2, sp); }

   /** "The" {@link M}.  */
   public final M Mdl = new M(0, 0, triv);

   /** A (trivially) fully parameterised model of integers &gt;0 where
       pr(n)&nbsp;=&nbsp;1/(n(n+1)).
       The UnParameterised model is {@link Int1}. */
   public class M extends Discretes.M
    { /** msg1 must be zero and sp must be triv.  */
      public M(double msg1, double msg2, Value sp)
       { super(msg1, msg2, sp); zeroTriv(msg1, sp); }//constructor

      /** Negative log likelihood of data-set ds[lo,hi) where
          {@link Int1#stats(Vector,int,int) ss=stats(ds,lo,hi)}. */
      public double nlLH(Value ss)
       { return sumNlPr((Vector) ss); }

      /** pr(n) = 1/(n(n+1)) so -log&nbsp;pr(n) equals
          log(n)&nbsp;+&nbsp;log(n+1). */
      public double nlPr_n(int n)                                  // nlPr_n(n)
       { return Math.log(n) + Math.log(n+1); }

      /** Not yet implemented (lazy).  */
      public int random_n()
       { throw RTE( ".random_n() is not yet implemented" ); }

    }//Int1.M class, the fully parameterised Model

 }//Int1 class, the (UnParameterised) UPModel

// (c) Lloyd Allison Wed 15 Feb 2017. Licence to 'Int1' is Gnu GPL.
