//       1         2         3         4         5         6         7
// 4567890123456789012345678901234567890123456789012345678901234567890123456789

package la.bioinformatics;

import la.util.*;
import la.la.Value;
import la.la.Type;
import la.maths.*;
import mml.*;
import static mml.MML.*;
// ----------------------------------------------------------------------------

/* Copyright (c) 2018 Lloyd Allison.

   Reference: 
     Lloyd Allison, "Coding Ockham's Razor", Springer, 2018.

   This file is free software: you can redistribute it and/or modify it
   under the terms of the GNU General Public License (GPL) as published
   by the Free Software Foundation, either version 3 of the License, or
   (at your option) any later version.

   This file is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this file. If not, see http://www.gnu.org/licenses/
 */
// ----------------------------------------------------------------------------

/** <I>This is experimental code, not a production application.</I>
    You are welcome to look, but consider it to be ephemeral.
    An Alignment of two sequences is seen primarily as a Vector of
    Inc_Or&nbsp;t1&nbsp;t2 where t1 is the element type of
    sequence&nbsp;1 and t2 is the element type of sequence&nbsp;2.
    Of course t1 and t2 may be the {@link Alignment.UPSame same}.  */
public abstract class Alignment extends Vector                             // *
 {
    /** The <KBD>int</KBD> tags for
        {@link Value.Inc_Or#LEFT         LEFT}&nbsp;|
        {@link Value.Inc_Or#RIGHT        RIGHT}
        |&nbsp;{@link Value.Inc_Or#RIGHT BOTH}. */
    private final static int
         LEFT  = Value.Inc_Or.LEFT,
         RIGHT = Value.Inc_Or.RIGHT,
         BOTH  = Value.Inc_Or.BOTH;

   /** elt(i) must be an {@link la.la.Value.Inc_Or}. */
   public abstract Value.Inc_Or elt(int i);                      // * elt(i)

   /** sel = LEFT | RIGHT | BOTH.  The Scannable elements of the
       sel-ected {@link #side(Value.Scannable,int) side} or sides (pairs)
       of this Alignment. */
   public Value.Scannable side(final int sel)
    { return side(this, sel); }

   /** The Scannable Int(n())s of this Alignment. That is
       of 0|1|2 for LEFT|RIGHT|BOTH repectively.  */
   public Value.Scannable flags()
    { return flags(this); }
   // .....................

   /** One way to create an {@link Alignment}, as a given. */
   public static Alignment algnmnt(final Value.Inc_Or [] Inc_Ors)
    { return new Alignment()
       { public int          nElts()    { return Inc_Ors.length; }
         public Value.Inc_Or elt(int i) { return Inc_Ors[i]; }
    }; }//algnmnt(Inc_Ors)
   // -------------------------------------------------------------------------

   /** Given two sequences, s1 and s2, and a SeriesModel of Alignments, m,
       return an optimal global Alignment of s1 and s2. It is required that
       m.sm3 incorporates simple costs &mdash; constant for each of insert,
       delete, match. */
   public static Alignment simple(Vector s1, Vector s2, UPSame.M m)
    { /** an3, trivial Analysis of []. */
      SeriesModel.Analysis an3 = m.sm3.analysis(Vector.ints(new int[0]));
      final double ins1 = an3.eltM().nlPr(new Value.Int(LEFT)),  // const!
                   ins2 = an3.eltM().nlPr(new Value.Int(RIGHT)), // -"-
                   diag = an3.eltM().nlPr(new Value.Int(BOTH));  // -"-
      FunctionModel x2y = m.x2y;  // e.g., a substitution CPT, say
      final int n1 = s1.nElts(), n2 = s2.nElts();
      byte   [] [] choice = new byte [n1+1] [n2+1];  // memo, ~ |s1|*|s2|
      double [] [] cost = new double [2] [n2+1];     // ~ 2*|s2|

      choice[0][0] = -1; // mark end
      cost[0][0] = 0;
      SeriesModel.Analysis a2 = m.smE.analysis(s2);
      for( int j = 1; j <= n2; j ++ ) // each col of top row
       { choice[0][j] = RIGHT;                         // top...
         cost[0][j] = cost[0][j-1] + ins2 + a2.nlPr(); // ...boundary
         a2.advance();
       }//j

      SeriesModel.Analysis a1 = m.smE.analysis(s1);  // -- a1
      for( int i = 1; i <= n1; i ++ ) // each row
       { Value  e1 = a1.eltD();
         double c1 = a1.nlPr();
         choice[i][0] = LEFT;                 // left...
         cost[1][0] = cost[0][0] + ins1 + c1; // ...boundary
         a2 = m.smE.analysis(s2);                        // -- a2
         for( int j = 1; j <= n2; j ++ ) // each col of row i
          { Value  e2 = a2.eltD();
            double c2 = a2.nlPr();
            double vc = cost[0][j]   + ins1 + c1,
                   hc = cost[1][j-1] + ins2 + c2,
                   dc = cost[0][j-1] + diag + c1 + x2y.condNlPr(e1,e2)
                                     + 1    + c2 + x2y.condNlPr(e2,e1);
            int ch;  double c;
            if( dc <= vc && dc <= hc ) { ch = BOTH;  c = dc; }
            else if( vc <= hc )        { ch = LEFT;  c = vc; }
            else                       { ch = RIGHT; c = hc; }
            choice[i][j] = (byte) ch; cost[1][j] = c;
            a2.advance();
          }//j
         double [] temp = cost[0]; cost[0] = cost[1]; cost[1] = temp; // swap
         a1.advance();
       }//i

      int l = 0, x = n1, y = n2, cxy = choice[x][y];
      while( cxy >= 0 ) // traceback to get length, l
       { l ++ ;
         if( cxy != RIGHT ) x -- ; // i.e,, LEFT  or BOTH
         if( cxy != LEFT  ) y -- ; // i.e., RIGHT or BOTH
         cxy = choice[x][y];
       }//while

      Value.Inc_Or [] a = new Value.Inc_Or [ l ];
      x = n1; y = n2;
      while( l > 0 ) // traceback to build Alignment
       { l -- ;
         switch( choice[x][y] )
          { case LEFT:  x -- ; a[l] = new Value.Inc_Or.Left (s1.elt(x)); break;
            case RIGHT: y -- ; a[l] = new Value.Inc_Or.Right(s2.elt(y)); break;
            case BOTH:  x -- ; y -- ;
                     a[l] = new Value.Inc_Or.Both(s1.elt(x), s2.elt(y)); break;
          }
       }//while
      return algnmnt( a );
    }//simple(s1,s2,m)

   /** Estimate a SeriesModel to suit the Alignment al. */
   public static UPSame.M a2m(Alignment al, Estimator e)
    { return (UPSame.M) e.ds2Model( Vector.values( new Value[]{ al } ) ); }

   public static Alignment EM(Vector s1, Vector s2, Estimator e)
    { Vector none = Vector.values(new Value[]{});
      Alignment al = null;
      UPSame.M m = (UPSame.M) e.ds2Model(none); // the Model
System.out.println( "start : m = " + m + ", " + m.msg1bits() + "+" + m.msg2bits() + " bits" );
      for(int i = 0; i < 4; i ++ )
       { al = simple(s1, s2, m);
System.out.println( i + ": al = " + al + ", " + m.nl2Pr(al) + " bits" );
         m = a2m(al, e);
System.out.println( i + " : m = " + m + ", " + m.msg1bits() + "+" + m.msg2bits() + "bits" );
       }//for i
      return al;
    }//EM(al,e)
   // -------------------------------------------------------------------------

   /** An UnParameterised SeriesModel of Alignments where both sequences are
       of the <I>same</I> element Type.  It is built from {@link #UPsm3 UPsm3},
       {@link #UPsmE UPsmE} and {@link #UPx2y UPx2y}.
       The fully parameterised SeriesModel is {@link M}.  */
   public static class UPSame extends UPSeriesModel.Length        // UPSame
    {
      /** UPsm3, the UnParameterised SeriesModel of the "flags",
          <NOBR>(LEFT | RIGHT | BOTH)<SUP>*</SUP></NOBR>.  */
      public final UPSeriesModel.Length UPsm3;

      /** UPsmE, the UnParameterised SeriesModel of elements. */
      public final UPSeriesModel UPsmE;

      /** UPx2y, the UnParameterised FunctionModel of l&rarr;r and r&rarr;l. */
      public final UPFunctionModel UPx2y;

      public UPSame(Value dp)
       { super(dp);
         final Tuple dpt = dp.isTuple(3);
         UPsm3 = (UPSeriesModel.Length) dpt.fst();
         UPsmE = (UPSeriesModel)        dpt.snd();
         UPx2y = (UPFunctionModel)      dpt.thrd();
       }//constructor

      /** {@link #UPsm3 UPsm3}.lenUPM() provides the UPModel of lengths. */
      public UPModel lenUPM() { return UPsm3.lenUPM(); }

      /** In this case, statistics ss <I>is</I> the data-set vs. */
      public Value stats(Vector vs, int lo, int hi) { return vs.slice(lo,hi); }

      public Value stats(boolean add, Value ss0, Value ss1)
       { Slice ss0v = (Slice) ss0, ss1v = (Slice) ss1;
         return add ? ss0v.addMore(ss1v) : ss0v.delete(ss1v);
       }

      /** Return the Estimator. Parameters ps, if any, are for
          <NOBR>({@link #UPsm3 UPsm3}, {@link #UPsmE UPsmE},
           {@link #UPx2y UPx2y})</NOBR> respectively. */
      public Est estimator(final Value ps)
       { return new Est(ps)
          { public M ss2Model(final Value ss)
             { Value.Tuple pst = ps.isTuple(3);
               Vector vs = (Vector) ss; // the data-set itself
               Vector [] f = new Vector[vs.nElts()], // flags
                         l = new Vector[vs.nElts()], // lefts
                         r = new Vector[vs.nElts()], // rights
                         m = new Vector[vs.nElts()]; // matched
               for( int i = 0; i < f.length; i ++ )  // each Alignment...
                { final Alignment a_i = (Alignment) vs.elt(i);
                  f[i] = Vector.fromScannable(a_i.flags());
                  l[i] = Vector.fromScannable(a_i.side(LEFT));
                  r[i] = Vector.fromScannable(a_i.side(RIGHT));
                  m[i] = Vector.fromScannable(a_i.side(BOTH));
                }
               Vector fv = Vector.values(f),
                      lv = Vector.values(l),
                      rv = Vector.values(r),
                      mv = Vector.values(m).append(); // NB. flattened
// System.out.println( " ZZZ{fv=" + fv + ", lv=" + lv + ", rv=" + rv + ", mv=" + mv + "} " );
               Tuple
                 sm3_mSp = UPsm3.estimator(pst.fst()).ds2ModelSp(fv),
                 smE_mSp = UPsmE.estimator(pst.snd()).ds2ModelSp(lv.append(rv)),
                 x2y_mSp = UPx2y.estimator(pst.thrd()).ds2ModelSp(
                             mv.append(mv.cols(new int[]{1, 0})) );
               SeriesModel   sm3 = (SeriesModel)   sm3_mSp.fst(),
                                 smE = (SeriesModel)   smE_mSp.fst();
               FunctionModel x2y = (FunctionModel) x2y_mSp.fst();
               M am = (M) sp2Model(0, 0, Value.triple(
                             sm3_mSp.snd(), smE_mSp.snd(), x2y_mSp.snd() ));
               double msg1 = sm3.msg1() + smE.msg1() + x2y.msg1(),
                      msg2 = am.nlLH(ss);
               return (M) am.asGiven(msg1, msg2);
             }//ss21Model(ss)
       }; }//estimator(ps)

      /** Given two part message lengths, msg1 and msg2, and
          statistical parameters, sp, return a {@link M M}.  */
      public M sp2Model(double msg1, double msg2, Value sp)
       { return new M(msg1, msg2, sp); }
      // ---------------------------------------

      /** The fully parameterised SeriesModel of Alignments. It is built
          from {@link #sm3 sm3}, {@link #smE smE} and {@link #x2y x2y}.
          The UnParameterised SeriesModel is {@link UPSame}.  */
      public class M extends UPSeriesModel.Length.M             // UPSame.M
       {
         /** sm3, the SeriesModel of
             <NOBR>(LEFT | RIGHT | BOTH)<SUP>*</SUP></NOBR>. */
         public final UPSeriesModel.Length.M sm3;
         /** smE, the SeriesModel of elements. */
         public final SeriesModel smE;
         /** x2y, the Function Model of l&rarr;r and r&rarr;l. */
         public final FunctionModel x2y;

          /** Statistical parameters, sp, are for <NOBR>({@link #UPsm3 UPsm3},
              {@link #UPsmE UPsmE}, {@link #UPx2y UPx2y})</NOBR>. */
         public M(double msg1, double msg2, Value sp)
          { super(msg1, msg2, sp);
            Tuple spt = sp.isTuple(3);
            sm3 = (UPSeriesModel.Length.M) UPsm3.apply(spt.fst());
            smE = (SeriesModel)            UPsmE.apply(spt.snd());
            x2y = (FunctionModel)          UPx2y.apply(spt.thrd());
          }//constructor

         public double nlLH(Value ss)
          { return sumNlPr((Vector) ss); }//nlLH(ss)

         /** {@link #sm3 sm3}.lenMdl() provides the Model of lengths. */
         public Model lenMdl() { return sm3.lenMdl(); }

         /** Separate out match/indel, Left elements, and Right elements
             from sv, do analyses of each, combine results.  */
         public Analysis analysis(final Scannable sv)
          { return new Analysis(sv)
             { Analysis ns = sm3.analysis(flags(sv)),  // (0|1|2)*
                        xs = smE.analysis(side(sv, LEFT )),
                        ys = smE.analysis(side(sv, RIGHT));

            // public Value eltD()   return ds.elt() -- from class Analysis
               public int      n() { return eltD().n(); } // 0|1|2

               /** eltM(), the Model of the pending data element. */
               public Model eltM()
                { return new Model.Defaults(0, 0, triv)
                   { /** The Model is a mixture of x;(y|x) and y;(x|y).  */
                     public double nlPr(Value xy)
                      { final Value.Inc_Or xOy = (Value.Inc_Or) xy;
                        final int n = xOy.n();
                        double ans = ns.eltM().nlPr(new Value.Int(n));
                        switch( n )
                         { case LEFT:  ans += xs.eltM().nlPr(xOy.elt(0));
                                       break;
                           case RIGHT: ans += ys.eltM().nlPr(xOy.elt(0));
                                       break;
                           case BOTH:
                              final Value x=xOy.elt(LEFT), y=xOy.elt(RIGHT);
                              ans += 1 + Maths.logSum(               // +mix...
                                xs.eltM().nlPr(x) + x2y.condNlPr(x,y), //x;y|x
                                ys.eltM().nlPr(y) + x2y.condNlPr(y,x));//y;x|y
                         }//switch
                        return ans;
                      }//nlPr(xy)

                     public Inc_Or random()
                      { switch( ns.eltM().random().n() )
                         { case LEFT:
                             return new Value.Inc_Or. Left(xs.eltM().random());
                           case RIGHT:
                             return new Value.Inc_Or.Right(ys.eltM().random());
                           default:
                             if( RNG.nextDouble() < 0.5 ) // 50:50
                              { Value x = xs.eltM().random(); // x; x->y
                                return new Value.Inc_Or.Both(
                                   x, x2y.condModel(x).random());
                              }
                             else
                              { Value y = ys.eltM().random(); // y; y->x
                                return new Value.Inc_Or.Both(
                                   x2y.condModel(y).random(), y);
                              }
                         }//switch
                      }//random()

                     public String toString()
                      { return ".eltM() at posn=" + ns.position() + " "; }
                }; }//eltM()

               public void advance()
                { if( n() != RIGHT ) xs.advance();
                  if( n() != LEFT  ) ys.advance();
                  ns.advance();
                  ds.advance();
                }//advance()
          }; }
       }//UPSame.M class, the SeriesModel

   }//UPSame class, the UnParameterised SeriesModel
   // .........................................................................

   /** sel = LEFT | RIGHT | BOTH. The elements on the <NOBR>sel-ected</NOBR>
       side or sides of the Scannable Value sv. */
   public static Value.Scannable side(final Value.Scannable sv, final int sel)
    { final int BOTH = Value.Inc_Or.BOTH;
      return new Value.Scannable()
       { public Series toSeries()
          { return  new Series()
             { int pos = 0;
               Series s = sv.toSeries();
               public boolean hasSome() { skip(); return s.hasSome(); }
               public int    position() { return pos; }
               public Value       elt()
                { skip();
                  final Value e = s.elt();
                  return sel == BOTH ? Value.pair(e.elt(0), e.elt(1))
                       : n() == BOTH ? e.elt(sel) : e.elt(0);
                }
               /** The wt() for BOTH gives (x,y) a wt of 0.5, 0.5 to x
                   and the same to y.  For (x,x) it gives 2*0.5 to x. */
               public double wt() { return n() == BOTH ? 0.5 : 1.0; }
               public void    advance()
                { pos ++ ; /* = 0, 1, ... */ s.advance(); }
               private int          n() { skip(); return s.elt().n(); }
               private void      skip() // skip lone wrong-side elts of s
                { while( s.hasSome() )
                   { final int f = s.elt().n();
                     if( f == BOTH || f == sel ) break;
                     s.advance();
                   }
                }//skip()
          }; }//toSeries()
    }; }//side(sv,sel)

   /** The Scannable Int(n())s of a Scannable of Inc_OR. */
   public static Value.Scannable flags(final Value.Scannable sv)
    { return new Value.Scannable()
       { public Series toSeries()
          { return  new Series()
             { Series s = sv.toSeries();
               public boolean hasSome() { return s.hasSome();  }
               public int    position() { return s.position(); }
               public Value       elt() { return new Value.Int(s.elt().n()); }
               public void    advance() { s.advance(); }
          }; }//toSeries()
    }; }//flags(sv)
   // -------------------------------------------------------------------------

  /** A very rudimentary test. */
  public static void main(String[] args)
    { System.out.println("-- test Alignment --");

      Type.Enum DNA = Type.DNA;
      Value A = DNA.int2value(0), C = DNA.int2value(1),
            G = DNA.int2value(2), T = DNA.int2value(3);

      Alignment empty = algnmnt(new Value.Inc_Or[]{ }),
             l = algnmnt(new Value.Inc_Or[]{ new Value.Inc_Or.Left(A) }),
             r = algnmnt(new Value.Inc_Or[]{ new Value.Inc_Or.Right(A) }),
             e = algnmnt(new Value.Inc_Or[]{ new Value.Inc_Or.Both(A,A) }),
             n = algnmnt(new Value.Inc_Or[]{ new Value.Inc_Or.Both(A,C) }),
             algnmnt = algnmnt( new Value.Inc_Or[]{
                new Value.Inc_Or.Both(A, A),
                new Value.Inc_Or.Both(C, C),
                new Value.Inc_Or.Both(G, G),
                new Value.Inc_Or.Both(T, T),
                new Value.Inc_Or.Left(A),
                new Value.Inc_Or.Both(A, A),
                new Value.Inc_Or.Both(C, C),
                new Value.Inc_Or.Both(G, G),
                new Value.Inc_Or.Both(T, T),
                new Value.Inc_Or.Right(G),
                new Value.Inc_Or.Both(G, G),
                new Value.Inc_Or.Both(T, T) } );
      Vector algnmnts = Vector.values(new Value[]{empty, l,r,e, algnmnt});
      System.out.println( "   algnmnt = " + algnmnt );

      UPModel upmLen = Poisson0,
              upm3   = new MultiState(Value.pair(Value.zero, Value.two)),
              upmDNA = new MultiState(Value.pair(A, T));

      Matrix subsMatrix = Matrix.doubles(new double[][]{
                                        {0.7, 0.1, 0.1, 0.1},
                                        {0.1, 0.7, 0.1, 0.1},
                                        {0.1, 0.1, 0.7, 0.1},
                                        {0.1, 0.1, 0.1, 0.7} });
      CPT.M dnaCPT = (CPT.M) new CPT(Value.triple(A, T, upmDNA))
                       .apply(subsMatrix);

      MultiState.M costM = (MultiState.M)
         new MultiState(Value.pair(Value.zero, Value.two))
             .apply(Vector.doubles(new double[]{0.1, 0.1, 0.8}));

      UPSame upsmAL = new UPSame( Value.triple(
            new UPSeriesModel.K(Value.pair( upmLen, costM.asUPModel() )),
            // ZZZ new UPSeriesModel.K(Value.pair( upmLen, upm3 )),
            new UPSeriesModel.K(Value.pair( KnownUPM, upmDNA )),
            dnaCPT.asUPModel() ) );
            // ZZZ new CPT(Value.triple(A, T, upmDNA)) ));

      Estimator est = upsmAL.estimator(
                          Value.triple(
                             Value.pair(Value.real(6), Value.triv),
                             Value.pair(Value.triv, Value.triv),
                             Value.triv) );
      UPSame.M alMdl = (UPSame.M) est.ds2Model( algnmnts );
      mml.Test.exercise( alMdl, algnmnts );

System.out.println( "---" );
      Vector s1 = Vector.values( new Value[]{A,C,A,G,  A,C,T,G,T,A} ),
             s2 = Vector.values( new Value[]{A,C,A,G,T,A,C,T,G,T} );
      Alignment al = simple(s1, s2, alMdl);
      System.out.println( "Alignment = " + al
                        + ", " + alMdl.nl2Pr(al) + " bits" );
System.out.println( "---" );

EM(s1, s2, est);

      System.out.println("-- done Alignment --");
    }//main

 }//Alignment class   http://www.allisons.org/ll/

// (c) Lloyd Allison, 6/2013, 23 Nov 2016. Licence to 'Alignment' is Gnu GPL.
